@@ -16,7 +16,6 @@
 #endif
 
 #define ARRAY_SIZE(arr) (sizeof(arr) / sizeof(*arr))
-#undef __PAX_UTILS_CLEANUP
 
 #include <assert.h>
 #include <ctype.h>
@@ -30,6 +29,7 @@
 #include <limits.h>
 #include <pwd.h>
 #include <regex.h>
+#include <sched.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -42,6 +42,10 @@
 #if !defined(__FreeBSD__) && !defined(__OpenBSD__)
 # include <alloca.h>
 #endif
+#if defined(__linux__)
+# include <sys/prctl.h>
+# include <linux/securebits.h>
+#endif
 #if defined(__GLIBC__) || defined(__UCLIBC__)
 # include <byteswap.h>
 # include <endian.h>
@@ -66,12 +70,20 @@
 # define __PAX_UTILS_DEFAULT_LD_CACHE_CONFIG ""
 #endif
 
+#undef PAX_UTILS_CLEANUP
 /* bounds checking code will fart on free(NULL) even though that
  * is valid usage.  So let's wrap it if need be.
  */
 #ifdef __BOUNDS_CHECKING_ON
 # define free(ptr) do { if (ptr) free(ptr); } while (0)
-# define __PAX_UTILS_CLEANUP
+# define PAX_UTILS_CLEANUP 1
+#endif
+/* LSAN (Leak Sanitizer) will complain about things we leak. */
+#ifdef __SANITIZE_ADDRESS__
+# define PAX_UTILS_CLEANUP 1
+#endif
+#ifndef PAX_UTILS_CLEANUP
+# define PAX_UTILS_CLEANUP 0
 #endif
 
 /* Few arches can safely do unaligned accesses */
@@ -124,7 +136,7 @@
 #endif
 
 #define _minmax(x, y, op) \
-	({ typeof(x) __x = (x); typeof(y) __y = (y); (__x op __y ? __x : __y); })
+	({ __typeof__(x) __x = (x); __typeof__(y) __y = (y); (__x op __y ? __x : __y); })
 #if !defined(min)
 # define min(x, y) _minmax(x, y, <)
 #endif
