@@ -1411,7 +1411,7 @@ scanelf_match_symname(elfobj *elf, char *found_sym, char **ret, size_t *ret_len,
 	*found_sym = 1;
 }
 
-static const char *scanelf_file_sym(elfobj *elf, char *found_sym)
+static char *scanelf_file_sym(elfobj *elf, char *found_sym)
 {
 	char *ret;
 	void *symtab_void, *strtab_void;
@@ -1458,17 +1458,23 @@ static const char *scanelf_file_sym(elfobj *elf, char *found_sym)
 		FIND_SYM(64)
 	}
 
-	if (be_wewy_wewy_quiet) return NULL;
+	if (be_wewy_wewy_quiet) {
+		free(ret);
+		return NULL;
+	}
 
 	if (*find_sym != '*' && *found_sym)
 		return ret;
+	else
+		free(ret);
 	if (be_quiet)
 		return NULL;
 	else
-		return " - ";
+		return xstrdup(" - ");
 
  break_out:
 	warnf("%s: corrupt ELF symbols", elf->filename);
+	free(ret);
 	return NULL;
 }
 
@@ -1583,6 +1589,7 @@ static int scanelf_elfobj(elfobj *elf)
 
 	/* dump all the good stuff */
 	for (i = 0; out_format[i]; ++i) {
+		char *allocated;
 		const char *out;
 		const char *tmp;
 		static char ubuf[sizeof(unsigned long)*2];
@@ -1591,7 +1598,7 @@ static int scanelf_elfobj(elfobj *elf)
 			continue;
 		}
 
-		out = NULL;
+		out = allocated = NULL;
 		be_wewy_wewy_quiet = (out_format[i] == '#');
 		be_semi_verbose = (out_format[i] == '+');
 		switch (out_format[++i]) {
@@ -1639,7 +1646,7 @@ static int scanelf_elfobj(elfobj *elf)
 		case 'i': out = scanelf_file_interp(elf, &found_interp); break;
 		case 'b': out = scanelf_file_bind(elf, &found_bind); break;
 		case 'S': out = scanelf_file_soname(elf, &found_soname); break;
-		case 's': out = scanelf_file_sym(elf, &found_sym); break;
+		case 's': out = allocated = scanelf_file_sym(elf, &found_sym); break;
 		case 'k': out = scanelf_file_sections(elf, &found_section); break;
 		case 'a': out = get_elfemtype(elf); break;
 		case 'I': out = get_elfosabi(elf); break;
@@ -1647,8 +1654,10 @@ static int scanelf_elfobj(elfobj *elf)
 		case 'Z': snprintf(ubuf, sizeof(ubuf), "%lu", (unsigned long)elf->len); out = ubuf; break;;
 		default: warnf("'%c' has no scan code?", out_format[i]);
 		}
-		if (out)
+		if (out) {
 			xstrcat(&out_buffer, out, &out_len);
+			free(allocated);
+		}
 	}
 
 #define FOUND_SOMETHING() \
@@ -1833,7 +1842,7 @@ static int scanelf_dirat(int dir_fd, const char *path)
 
 		len = strlen(dentry->d_name);
 		if (len + pathlen + 1 >= sizeof(buf)) {
-			warnf("Skipping '%s%s': len > sizeof(buf); %zu > %zu\n",
+			warnf("Skipping '%s%s': len > sizeof(buf); %zu > %zu",
 			      path, dentry->d_name, len + pathlen + 1, sizeof(buf));
 			continue;
 		}
@@ -2264,7 +2273,10 @@ static int parseargs(int argc, char *argv[])
 			break;
 		case 'F': {
 			if (out_format) warn("You prob don't want to specify -F twice");
-			out_format = optarg;
+			if (PAX_UTILS_CLEANUP)
+				out_format = xstrdup(optarg);
+			else
+				out_format = optarg;
 			break;
 		}
 		case 'z': {
@@ -2460,6 +2472,10 @@ static int parseargs(int argc, char *argv[])
 	}
 	if (be_verbose > 2) printf("Format: %s\n", out_format);
 
+	/* Now lock down the pidns since we know whether we'll be forking. */
+	if (!show_textrels || !be_verbose)
+		security_init_pid();
+
 	/* now lets actually do the scanning */
 	if (load_cache_config)
 		load_ld_cache_config(__PAX_UTILS_DEFAULT_LD_CACHE_CONFIG);
@@ -2478,26 +2494,26 @@ static int parseargs(int argc, char *argv[])
 		ret = scanelf_dir(search_path);
 	}
 
-#ifdef __PAX_UTILS_CLEANUP
-	/* clean up */
-	xarrayfree(ldpaths);
-	xarrayfree(find_sym_arr);
-	xarrayfree(find_lib_arr);
-	xarrayfree(find_section_arr);
-	free(find_sym);
-	free(find_lib);
-	free(find_section);
-	{
-		size_t n;
-		regex_t *preg;
-		array_for_each(find_sym_regex_arr, n, preg)
-			regfree(preg);
-		xarrayfree(find_sym_regex_arr);
-	}
+	if (PAX_UTILS_CLEANUP) {
+		/* clean up */
+		xarrayfree(ldpaths);
+		xarrayfree(find_sym_arr);
+		xarrayfree(find_lib_arr);
+		xarrayfree(find_section_arr);
+		free(find_sym);
+		free(find_lib);
+		free(find_section);
+		{
+			size_t n;
+			regex_t *preg;
+			array_for_each(find_sym_regex_arr, n, preg)
+				regfree(preg);
+			xarrayfree(find_sym_regex_arr);
+		}
 
-	if (ldcache != 0)
-		munmap(ldcache, ldcache_size);
-#endif
+		if (ldcache != 0)
+			munmap(ldcache, ldcache_size);
+	}
 
 	return ret;
 }
@@ -2544,30 +2560,27 @@ static void parseenv(void)
 	qa_wx_load = get_split_env("QA_WX_LOAD");
 }
 
-#ifdef __PAX_UTILS_CLEANUP
 static void cleanup(void)
 {
+	if (!PAX_UTILS_CLEANUP)
+		return;
+
 	free(out_format);
 	free(qa_textrels);
 	free(qa_execstack);
 	free(qa_wx_load);
 }
-#endif
 
 int main(int argc, char *argv[])
 {
 	int ret;
+	security_init(true);
 	if (argc < 2)
 		usage(EXIT_FAILURE);
 	parseenv();
 	ret = parseargs(argc, argv);
 	fclose(stdout);
-#ifdef __PAX_UTILS_CLEANUP
 	cleanup();
-	warn("The calls to add/delete heap should be off:\n"
-	     "\t- 1 due to the out_buffer not being freed in scanelf_fileat()\n"
-	     "\t- 1 per QA_TEXTRELS/QA_EXECSTACK/QA_WX_LOAD");
-#endif
 	return ret;
 }
 
